// -*- C++ -*-
/*!
 * @file  VelocityManager.cpp
 * @brief VelocityManager
 * @date $Date$
 *
 * $Id$
 */

#include "VelocityManager.h"

// Module specification
// <rtc-template block="module_spec">
static const char* velocitymanager_spec[] =
  {
    "implementation_id", "VelocityManager",
    "type_name",         "VelocityManager",
    "description",       "VelocityManager",
    "version",           "1.0.0",
    "vendor",            "YukiMurata",
    "category",          "Controller",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    ""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
VelocityManager::VelocityManager(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_flagIn("flag", m_flag),
    m_idIn("id", m_id),
    m_positionIn("position", m_position),
    m_velocityOut("velocity", m_velocity),
    m_targetJointAngleOut("targetJointAngle", m_targetJointAngle),
    m_orderOut("order", m_order)

    // </rtc-template>
{
}

/*!
 * @brief destructor
 */
VelocityManager::~VelocityManager()
{
}



RTC::ReturnCode_t VelocityManager::onInitialize()
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  addInPort("flag", m_flagIn);
  addInPort("id", m_idIn);
  addInPort("position", m_positionIn);
  
  // Set OutPort buffer
  addOutPort("velocity", m_velocityOut);
  addOutPort("targetJointAngle", m_targetJointAngleOut);
  addOutPort("order", m_orderOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

  // <rtc-template block="bind_config">
  // </rtc-template>
  
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t VelocityManager::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t VelocityManager::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t VelocityManager::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t VelocityManager::onActivated(RTC::UniqueId ec_id)
{
	m_velocity.data.vx = 0;
	m_velocity.data.vy = 0;
	m_velocity.data.va = 0;

	m_targetJointAngle.data.length(14);
	for (int i = 0; i<14; i++)
	{
		m_targetJointAngle.data[i] = 0.0;
	}
	m_targetJointAngle.data[2] = 1.5;
	m_targetJointAngle.data[8] = 1.5;
	m_targetJointAngle.data[10] = 1.5;
	m_targetJointAngle.data[13] = 0.99;
	flag = 9;

	m_velocityOut.write();
	m_targetJointAngleOut.write();

	std::cout << "standby" << std::endl;

  return RTC::RTC_OK;
}


RTC::ReturnCode_t VelocityManager::onDeactivated(RTC::UniqueId ec_id)
{
	m_velocity.data.vx = 0;
	m_velocity.data.va = 0;

	for (int i = 0; i<14; i++)
	{
		m_targetJointAngle.data[i] = 0.0;
	}
	m_targetJointAngle.data[2] = -1.5;
	m_targetJointAngle.data[8] = -1.5;
	m_targetJointAngle.data[13] = 0.99;

	m_velocityOut.write();
	m_targetJointAngleOut.write();
	flag = 9;

	std::cout << "finish" << std::endl;

  return RTC::RTC_OK;
}


RTC::ReturnCode_t VelocityManager::onExecute(RTC::UniqueId ec_id)
{
	if (m_flagIn.isNew())
	{
		m_flagIn.read();
		if (m_flag.data == 0)
		{
			flag = 0;
			std::cout << "standby by flag" << std::endl;
		}
		else if (m_flag.data == 1)
		{
			flag = 1;
			std::cout << "preparing for detecting " << std::endl;

			m_velocity.data.va = 0.3;
			m_velocityOut.write();
			Sleep(8000);
			m_velocity.data.va = 0;
			m_velocityOut.write();
			Sleep(3000);

			m_velocity.data.vx = 0;
			m_velocity.data.va = 0;

			m_targetJointAngle.data.length(14);
			for (int i = 0; i < 14; i++)
			{
				m_targetJointAngle.data[i] = 0.0;
			}
			m_targetJointAngle.data[2] = 1.5;
			m_targetJointAngle.data[8] = 1.5;
			m_targetJointAngle.data[10] = 1.5;

			m_velocityOut.write();
			m_targetJointAngleOut.write();
		}
		else if (m_flag.data == 2)
		{
			std::cout << "starting selfie ..." << std::endl;
			flag = 2;
			m_velocity.data.va = -0.3;
			m_velocityOut.write();
			Sleep(8000);
			m_velocity.data.va = 0;
			m_velocityOut.write();
			Sleep(3000);

			m_targetJointAngle.data[0] = -0.7;
			m_targetJointAngle.data[1] = 0.04;
			m_targetJointAngle.data[2] = -0.25;
			m_targetJointAngle.data[3] = 0.19;
			m_targetJointAngle.data[4] = -0.5;
			m_targetJointAngle.data[5] = -2.72;
			m_targetJointAngle.data[6] = 0;
			m_targetJointAngle.data[7] = 1.1;
			m_targetJointAngle.data[8] = 1.2;
			m_targetJointAngle.data[9] = -0.53;
			m_targetJointAngle.data[10] = 2.42;
			m_targetJointAngle.data[11] = 0.25;
			m_targetJointAngle.data[12] = 0.27;
			m_targetJointAngle.data[13] = -0.99;

			m_targetJointAngleOut.write();
		}
	}

	if (flag == 9)
	{
		std::cout << "input 1 for start" << std::endl;
		std::cin >> a;
		m_targetJointAngle.data[13] = -0.99;
		flag = 1;
	}

	if (flag == 0) //standby
	{

	}

	if (flag == 1) //search
	{
		m_idIn.read();
		m_positionIn.read();
		if (m_id.data == 1)
		{
			if (m_position.data.x != previousx && m_position.data.y != previousy)
			{
				scorex = 133 - m_position.data.x;
				scorey = m_position.data.y - 100;
				m_targetJointAngle.data[0] += scorex * 0.002;
				m_targetJointAngle.data[1] += scorey * 0.002;
				if (m_targetJointAngle.data[0] > 1)
				{
					m_targetJointAngle.data[0] = 1;
				}
				else if (m_targetJointAngle.data[0] < -1)
				{
					m_targetJointAngle.data[0] = -1;
				}
				if (m_targetJointAngle.data[1] > 0.45)
				{
					m_targetJointAngle.data[1] = 0.45;
				}
				else if (m_targetJointAngle.data[1] < -0.4)
				{
					m_targetJointAngle.data[1] = -0.4;
				}
				previousx = m_position.data.x;
				previousy = m_position.data.y;
			}

			if ((m_position.data.x >= 125 && m_position.data.x <= 136) && (m_position.data.y >= 80 && m_position.data.y <= 103))
			{
				counter++;
			}

		}
		else
		{
			counter = 0;
		}

		m_targetJointAngleOut.write();
		m_velocityOut.write();
		std::cout << flag << "|ID, " << m_id.data << "|POS, " << m_position.data.x << "\t," << m_position.data.y << "\t|Vel, " << m_velocity.data.va << "\t|Ang, " << m_targetJointAngle.data[0] << "\t" << m_targetJointAngle.data[1] << "\t|X,Y,  " << scorex << "\t" << scorey << std::endl;

		if (counter > 200)//matching
		{
			m_order.data = 2;
			m_orderOut.write();
			counter = 0;
			flag = 0;
			std::cout << "waiting for Permission" << std::endl;
			previousx = 0;
			previousy = 0;
		}
	}

	if (flag == 2) //takeapicture
	{
		m_positionIn.read();
		m_idIn.read();
		std::cout << "adjusting..." << std::endl;
		if (m_id.data == 1)
		{
			std::cout << "Recognition" << std::endl;
			if (m_position.data.x > 267)
			{
				m_targetJointAngle.data[12] += 0.5;
			}
			else
			{
				m_targetJointAngle.data[9] += 0.5;
			}

			/*			if (m_position.data.x != previousx && m_position.data.y != previousy)
						{
						if ()
						{
						if (m_position.data.x > 386)
						{
						m_targetJointAngle.data[12] += 0.1;
						}
						else if (m_position.data.x < 148)
						{
						m_targetJointAngle.data[12] -= 0.1;
						}
						}
						else if()
						{
						if (m_position.data.x > 386)
						{
						m_targetJointAngle.data[9] -= 0.1;
						}
						else if (m_position.data.x < 148)
						{
						m_targetJointAngle.data[9] += 0.1;
						}
						}*/


			if (true)
			{
				m_order.data = 2;
				m_orderOut.write();
				previousx = 0;
				previousy = 0;
				flag = 0;
				std::cout << "standby for photo" << std::endl;
			}
		}

	}
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t VelocityManager::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t VelocityManager::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t VelocityManager::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t VelocityManager::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t VelocityManager::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void VelocityManagerInit(RTC::Manager* manager)
  {
    coil::Properties profile(velocitymanager_spec);
    manager->registerFactory(profile,
                             RTC::Create<VelocityManager>,
                             RTC::Delete<VelocityManager>);
  }
  
};


