// -*- C++ -*-
/*!
 * @file  FaceDetect.cpp
 * @brief Face Detection component
 * @date $Date$
 *
 * $Id$
 */

#include "FaceDetect.h"

// Module specification
// <rtc-template block="module_spec">
static const char* facedetect_spec[] =
  {
    "implementation_id", "FaceDetect",
    "type_name",         "FaceDetect",
    "description",       "Face Detection component",
    "version",           "1.0.0",
    "vendor",            "AIST",
    "category",          "Category",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.haarcascade", "../../data/haarcascades/haarcascade_frontalface_alt2.xml",
    "conf.default.downscale", "1.2",
    "conf.default.min_object_width", "30",
    "conf.default.min_object_height", "30",
    // Widget
    "conf.__widget__.haarcascade", "text",
    // Constraints
    ""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
FaceDetect::FaceDetect(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_imageIn("image", m_image),
    m_detectedImageOut("detectedImage", m_detectedImage),
    m_positionOut("position", m_position),
    m_facesOut("faces", m_faces)

    // </rtc-template>
{
}

/*!
 * @brief destructor
 */
FaceDetect::~FaceDetect()
{
}



RTC::ReturnCode_t FaceDetect::onInitialize()
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  addInPort("image", m_imageIn);
  
  // Set OutPort buffer
  addOutPort("detectedImage", m_detectedImageOut);
  addOutPort("position", m_positionOut);
  addOutPort("faces", m_facesOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("haarcascade", m_haarcascade, "../../data/haarcascades/haarcascade_frontalface_alt2.xml");
  bindParameter("downscale", m_downscale, "1.2");
  bindParameter("min_object_width", m_min_object_width, "30");
  bindParameter("min_object_height", m_min_object_height, "30");
 
  // </rtc-template>
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t FaceDetect::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t FaceDetect::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t FaceDetect::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t FaceDetect::onActivated(RTC::UniqueId ec_id)
{
  // C[Wp̏
  m_imageBuff = NULL;
  m_cvMStr = NULL;
  m_cvHCC = NULL;

  // OutPort̉ʃTCY̏
  m_detectedImage.width = 0;
  m_detectedImage.height = 0;

  // oɕKvȃXg[Wpӂ
  m_cvMStr = cvCreateMemStorage(0);
  cvClearMemStorage(m_cvMStr);

  try {
	// ʊ猟o̓ǂݍ
	m_cvHCC = (CvHaarClassifierCascade*)cvLoad(m_haarcascade.c_str());
  }
  catch (...) {
	std::cerr << m_haarcascade << " Couldn't open." << std::endl;
	return RTC::RTC_ERROR;
  }

  return RTC::RTC_OK;
}


RTC::ReturnCode_t FaceDetect::onDeactivated(RTC::UniqueId ec_id)
{
  if(m_imageBuff != NULL)
    cvReleaseImage(&m_imageBuff);

  if(m_cvMStr != NULL)
	cvReleaseMemStorage(&m_cvMStr);

  if(m_cvHCC != NULL)
	cvReleaseHaarClassifierCascade(&m_cvHCC);
	
  return RTC::RTC_OK;
}


RTC::ReturnCode_t FaceDetect::onExecute(RTC::UniqueId ec_id)
{

  if (! m_imageIn.isNew())
	return RTC::RTC_OK;

  m_imageIn.read();
	
  if (m_image.width != m_detectedImage.width ||
	  m_image.height != m_detectedImage.height) {
	m_detectedImage.width  = m_image.width;
	m_detectedImage.height = m_image.height;

	if (m_imageBuff != NULL) {
	  cvReleaseImage(&m_imageBuff);
	}
	m_imageBuff = cvCreateImage(cvSize(m_image.width, m_image.height),
								IPL_DEPTH_8U, 3);
  }

  memcpy(m_imageBuff->imageData,(void *)&(m_image.pixels[0]),m_image.pixels.length());
  
  CvMemStorage* c_memStr = cvCreateChildMemStorage(m_cvMStr);

  // o󂯎邽߂̃V[PXpӂ
  CvSeq* face;

  // 摜猟oΏۂ̏擾
  if (m_cvHCC == NULL) {
	std::cerr << "m_cvHCC is NULL!!!!" << std::endl;
  }

  IplImage* gray = cvCreateImage(cvSize(m_imageBuff->width, m_imageBuff->height),
                                 IPL_DEPTH_8U, 1);
  IplImage* detect_frame = cvCreateImage(cvSize((m_imageBuff->width / m_downscale), (m_imageBuff->height / m_downscale)),
                                 IPL_DEPTH_8U, 1);
  cvCvtColor(m_imageBuff, gray, CV_BGR2GRAY);
  cvResize(gray, detect_frame, CV_INTER_LINEAR);
  cvEqualizeHist(detect_frame, detect_frame);
  face = cvHaarDetectObjects(detect_frame, m_cvHCC, c_memStr, m_downscale, 2, CV_HAAR_DO_CANNY_PRUNING,cvSize(m_min_object_width,m_min_object_height));

  cvReleaseMemStorage(&c_memStr);

  for (int i = 0; i < face->total; ++i) {
	// o񂩂̈ʒu擾
	CvRect* faceRect = (CvRect*)cvGetSeqElem(face, i);

	// 擾̈ʒuɊÂA``s
	cvRectangle(m_imageBuff,
				cvPoint(faceRect->x*m_downscale, faceRect->y*m_downscale),
				cvPoint((faceRect->x + faceRect->width)*m_downscale, (faceRect->y + faceRect->height)*m_downscale),
				CV_RGB(255, 0 ,0),
				3, CV_AA);
    m_position.data.x = faceRect->x;
    m_position.data.y = faceRect->y;
  }
  m_faces.data = face->total;

  //cvClearSeq(face);
 
  // 摜f[^̃TCY擾
  int len = m_imageBuff->nChannels * m_imageBuff->width * m_imageBuff->height;
  m_detectedImage.pixels.length(len);
 
  // 摜f[^OutPortɃRs[
  memcpy((void *)&(m_detectedImage.pixels[0]),m_imageBuff->imageData,len);

  m_detectedImageOut.write();
  m_facesOut.write();  
  m_positionOut.write();
  cvReleaseImage(&gray);
  cvReleaseImage(&detect_frame);

  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t FaceDetect::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t FaceDetect::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t FaceDetect::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t FaceDetect::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t FaceDetect::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void FaceDetectInit(RTC::Manager* manager)
  {
    coil::Properties profile(facedetect_spec);
    manager->registerFactory(profile,
                             RTC::Create<FaceDetect>,
                             RTC::Delete<FaceDetect>);
  }
  
};

